#include "StdAfx.h"
#include "game.h"
#include "object.h"
#include "enemy.h"
#include "bullet.h"
#include "menues.h"
#include <fstream>
#include <list>
using namespace std;

typedef list<object*> objList;
typedef list<object*>::iterator objIter;
typedef list<enemy*> enemList;
typedef list<enemy*>::iterator enemIter;
typedef list<bullet*> bulList;
typedef list<bullet*>::iterator bulIter;

namespace obt
{
	enum objBlueprintTypes
	{
		depotFuel = 0,
		depotAmmo = 1,
		station = 2,
		actionTime = 3
	};
}

sound mg, rocket;
bool game(int level, int &points)
{
	//Load the level
	ifstream lvl((string("maps/map") + i2str(level) + ".map").c_str(), ios::binary);
	if(!lvl)
	{
		error::mapLoadError(string("maps/map") + i2str(level) + ".map", "File does not exist!");
	}
	//Load level length
	int length, sections;
	lvl.read(reinterpret_cast<char*>(&length), sizeof(length));
	sections = length / 20;
	
	//Load some other stuff
	int timeLimit, ammo[3];
	lvl.read(reinterpret_cast<char*>(&timeLimit), sizeof(timeLimit));
	lvl.read(reinterpret_cast<char*>(ammo), sizeof(int) * 3);
	timeLimit *= 32;
	
	//Load background image
	string bgPath;
	getline(lvl, bgPath, char(0));
	asciibmp bg(bgPath);
	asciibmp *rails = new asciibmp("gfx/track");
	rails->draw(bg, 0, 37);
	delete rails;
	rails = 0;

	//Load blueprints
	object *oblueprints[255];
	enemy *eblueprints[255];
	for(int i = 0; i < 255; ++i)
	{
		oblueprints[i] = 0;
		eblueprints[i] = 0;
	}
	ifstream objFile("objects/objectsindex.txt");
	while(true)
	{
		//Read the character that corresponds to the object
		unsigned char key = objFile.get();
		//Ignore the whitespace between key and value
		objFile.get();
		//Read the rest of the line as path to the object file
		string nextPath;
		getline(objFile, nextPath, '\n');
		if(nextPath.length() == 0)
			break;
		oblueprints[key] = new object(nextPath);
		if(objFile.eof())
			break;
	}
	objFile.close();
	ifstream eneFile("objects/enemiesindex.txt");
	while(true)
	{
		//Read the character that corresponds to the object
		unsigned char key = eneFile.get();
		//Ignore the whitespace between key and value
		eneFile.get();
		//Read the rest of the line as path to the object file
		string nextPath;
		getline(eneFile, nextPath, '\n');
		if(nextPath.length() == 0)
			break;
		eblueprints[key] = new enemy(nextPath);
		if(eneFile.eof())
			break;
	}
	eneFile.close();

	//Load sections and enemies/objects
	objList objects;
	objList events;
	enemList enemies;
	oblueprints[obt::depotFuel] = new object("objects/fueldepot.txt");
	oblueprints[obt::depotAmmo] = new object("objects/ammodepot.txt");
	oblueprints[obt::station] = new object("objects/station.txt");
	for(int i = 0; i < sections; ++i)
	{
		int objCount = lvl.get(), enemyCount = lvl.get();
		if(objCount == 255 && enemyCount == 255)
		{
			unsigned short type;
			lvl.read(reinterpret_cast<char*>(&type), sizeof(type));
			if(type < 20001)
				events.push_back(new object(*(oblueprints[obt::depotFuel]), i * 20, type));
			else if(type < 32001)
				events.push_back(new object(*(oblueprints[obt::depotAmmo]), i * 20, type));
			else if(type < 48001)
				events.push_back(new object(*(oblueprints[obt::station]), i * 20, type));
			else if(type < 65501)
				events.push_back(new object(*(oblueprints[obt::actionTime]), i * 20, type));
		}
		else
		{
			for(int j = 0; j < objCount; ++j)
			{
				unsigned char type = lvl.get();
				int posX = i * 20 + lvl.get();
				int posY = lvl.get();
				objects.push_back(new object(*(oblueprints[type]), posX, posY));
			}
			for(int j = 0; j < enemyCount; ++j)
			{
				unsigned char type = lvl.get();
				int posX = i * 20 + lvl.get();
				int posY = lvl.get();
				int direction = lvl.get();
				enemies.push_back(new enemy(*(eblueprints[type]), posX, posY, direction));
			}
		}
	}
	lvl.close();

	//Start intro animation
	asciibmp train("gfx/train");
	char cannonGfx[8] = { '|', '/', '-', '\\', '|', '/', '-', '\\'};
	int cannonDrawPos[8][2] = { {1, 0}, {2, 0}, {2, 1}, {2, 2}, {1,2}, {0,2}, {0,1}, {0,0} };
	int weaponPos = 10, aim = 2;
	{
		int pos = 70, speed = 8, speedCounter = 0;
		timerCounter = 0;
		while(pos)
		{
			if(timerCounter > 0)
			{
				speedCounter += 3;
				if(speedCounter >= speed)
				{
					--pos;
					speedCounter -= speed;
				}
				clear_to_color(global::dbuffer, colors::black);
				bg.draw(global::abuffer, 0, 0);
				train.draw(global::abuffer, 30 - pos, 33);
				global::abuffer->setPixel(33 + weaponPos - pos, 34, 'O', colors::white);
				global::abuffer->setPixel(32 + weaponPos + cannonDrawPos[aim][0] - pos, 33 + cannonDrawPos[aim][1], cannonGfx[aim], colors::white);
				global::abuffer->draw(global::dbuffer);
				show();
				if(keyStates[KEY_ESC])
					pos = 0;
				--timerCounter;
			}
			rest(1);
		}
	}

	//Start game logic
	int pos = 0, speed = 10, desiredSpeed = 10, desiredSpeedCounter = 0, speedCounter = 0;
	const string dispSpeed[15] = { "0", "3.6", "7.2", "14.4", "19.08", "21.6", "24.84", "28.8", "34.56", "43.2", "49.32", "57.6", "68.4", "86.4", "115.2" };
	string stopNames[4] = { "Fuel depot", "Ammo depot", "Station", "Ambush" };
	const int speedIncrement[15] = { 0, 96, 48, 24, 18, 16, 14, 12, 10, 8, 7, 6, 5, 4, 3 },
		fuelUse[15] = { 0, 1, 3, 6, 8, 10, 20, 45, 60, 90, 110, 130, 140, 250, 350 },
		breakingDist[7] = { 34, 50, 67, 81, 105, 129, 164 };
	bool run = true, levelFinished = false;
	int playerHealth = 1000, playerFuel = 320000;
	int enemiesKilled = 0, stationsMissed = 0, oldScore = points;
	bulList bullets;
	object playerObject;
	objIter nextStop = events.begin();
	playerObject.set_pic(&train);
	playerObject.set_pos(30, 33);
	if(!mg.isActive())
		mg.load("sfx/shot.wav");
	if(!rocket.isActive())
		rocket.load("sfx/rocket.wav");
	while(run)
	{
		if(timerCounter > 0)
		{
			--timeLimit;
			if(speedIncrement[speed])
			{
				playerFuel -= fuelUse[speed];
				speedCounter += 3;
				while(speedCounter >= speedIncrement[speed])
				{
					++pos;
					speedCounter -= speedIncrement[speed];
					if(pos + 100 >= length)
					{
						levelFinished = true;
						run = false;
						break;
					}
					playerObject.set_pos(pos + 30, 33);
				}
			}
			else
			{
				speedCounter = 0;
			}
			if(keyStates[KEY_ESC])
				 run = false;
			if(keyStates[KEY_A] && desiredSpeed < 14)
				++desiredSpeed;
			if(keyStates[KEY_B] && desiredSpeed > 0)
				--desiredSpeed;
			if(desiredSpeed != speed)
			{
				if(desiredSpeedCounter < 48)
					++desiredSpeedCounter;
				if(desiredSpeedCounter >= 48)
				{
					desiredSpeedCounter = 0;
					if(desiredSpeed > speed)
						++speed;
					//Breaking is faster than acceleratig so you can break 2 units/second
					if(desiredSpeed < speed)
					{
						--speed;
						if(desiredSpeed < speed)
							--speed;
					}
				}
			}
			else
				desiredSpeedCounter = 0;
			if(keyStates[KEY_LEFT] && weaponPos > 0)
				--weaponPos;
			if(keyStates[KEY_RIGHT] && weaponPos < 19)
				++weaponPos;
			if(keyStates[KEY_UP])
				++aim %= 8;
			if(keyStates[KEY_DOWN])
			{
				--aim;
				if(aim < 0)
					aim = 7;
			}
			if(keyStates[KEY_SPACE] && ammo[0] > 0)
			{
				--ammo[0];
				bullet *newBul = new bullet;
				newBul->set_facing(aim);
				newBul->set_owner(0);
				
				newBul->set_speed(1);
				newBul->set_damage(10);
				newBul->set_range(100);
				
				newBul->set_pos(pos + 33 + weaponPos, 34);
				bullets.push_back(newBul);
				mg.play(0, 255);
			}
			if(keyStates[KEY_ALT] && ammo[1] > 0 && aim != 2)
			{
				--ammo[1];
				bullet *newBul = new bullet;
				newBul->set_facing(aim);
				newBul->set_owner(0);
				
				newBul->set_speed(2);
				newBul->set_damage(25);
				newBul->set_range(150);
				
				newBul->set_pos(pos + 33 + weaponPos, 34);
				bullets.push_back(newBul);
				rocket.play(0, 255);
			}
			if(keyStates[KEY_F1])
				showManual();
			
			if(playerHealth <= 0 || playerFuel <= 0)
			{
				run = false;
				break;
			}
			
			nextStop = events.begin();
			bool breakNow = false;
			int nextStopType = 0;
			if(nextStop != events.end())
			{
				object *curObj = *nextStop;
				int distance = (curObj->get_posx() + curObj->get_pic()->getW()) - (pos + 70);
				int stopType = curObj->get_posy();
				if(stopType < 20001)
					nextStopType = 0;
				else if(stopType < 32001)
					nextStopType = 1;
				else if(stopType < 48001)
					nextStopType = 2;
				else if(stopType < 65501)
					nextStopType = 3;
				
				if(speed > 7 && distance <= breakingDist[speed - 8] && distance > 0)
					breakNow = true;
				
				if((distance == 2 || distance == 1) && speed <= 7)
				{
					speed = desiredSpeed = 0;
					pos = curObj->get_posx() + curObj->get_pic()->getW() - 70;
				}
				else if((distance == 2 || distance == 1) && nextStopType == 2)
				{
					++stationsMissed;
					pos = curObj->get_posx() + curObj->get_pic()->getW() - 70;
				}
				if(distance == 0)
				{
					if(nextStopType == 0)
					{
						if(playerFuel < 320000)
						{
							playerFuel += 500;
							if(playerFuel > 320000)
								playerFuel = 320000;
						}
					}
					else if(nextStopType == 1)
					{
						stopType -= 20001;
						static const int maxAmmo[5][3] = { {500, 0, 0}, { 500, 10, 0 },
							{ 500, 10, 10 }, { 750, 20, 10 }, { 1000, 30, 10 } };
						for(int i = 0; i < 3; ++i)
						{
							if(ammo[i] < maxAmmo[stopType][i])
							{
								++ammo[i];
								if(!i)
									ammo[i] += 4;
							}
						}
					}
					else if(nextStopType == 2)
					{
						if(playerHealth < 750)
							++playerHealth;
					}
					else if(nextStopType == 3)
					{
					}
				}
			}
			
			for(enemIter i = enemies.begin(); i != enemies.end(); ++i)
			{
				enemy *curObj = *i;
				if(curObj->get_posx() + curObj->get_pic()->getW() < pos)
				{
					delete curObj;
					i = enemies.erase(i);
					//Go back by one because it will go forward just when continue is called
					--i;
					continue;
				}
				if(curObj->get_posx() < pos + 100)
				{
					bullet *newBullet = curObj->do_ai(pos + 30);
					if(newBullet)
					{
						bullets.push_back(newBullet);
					}
				}
			}
			
			for(bulIter i = bullets.begin(); i != bullets.end(); ++i)
			{
				bullet *curObj = *i;
				bool destroy = false;
				if(!curObj->logic())
				{
					destroy = true;
				}
				if(!destroy && curObj->get_owner() == 1)
				{
					if(curObj->collides(&playerObject))
					{
						destroy = true;
						playerHealth -= curObj->get_damage();
					}
				}
				else if(!destroy && curObj->get_owner() == 0)
				{
					for(enemIter j = enemies.begin(); j != enemies.end(); ++j)
					{
						enemy *curEnemObj = *j;
						if(curObj->collides(curEnemObj))
						{
							curEnemObj->set_health(curEnemObj->get_health() - curObj->get_damage());
							if(curEnemObj->is_dead())
							{
								points += 5;
								++enemiesKilled,
								delete curEnemObj;
								enemies.erase(j);
							}
							destroy = true;
							break;
						}
					}
				}
				if(destroy)
				{
					delete curObj;
					i = bullets.erase(i);
					//Go back by one because it will go forward just when continue is called
					--i;
					continue;
				}
			}
			
			//Drawing code
			clear(global::dbuffer);
			global::abuffer->clear(colors::black);
			
			//Draw GUI
			global::abuffer->hline('=', 0, 67, 99, colors::white);
			global::abuffer->textwrite("Speed", 0, 68, colors::white);
			global::abuffer->hline('-', 0, 69, 14, makecol(0xCC, 0xCC, 0xCC));
			global::abuffer->setPixel(desiredSpeed, 69, '|', colors::white);
			global::abuffer->textwritecenter(dispSpeed[speed] + " km/h", 7, 70, colors::white);
			
			global::abuffer->textwrite("Fuel: " + i2str(playerFuel / 3200) + "%", 0, 72, colors::white);
			
			global::abuffer->textwriteright("Health: " + i2str(playerHealth / 10) + "%", 98, 72, colors::white);
			global::abuffer->hline('|', 97 - playerHealth / 100, 74, 97, colors::green);
			
			global::abuffer->textwrite("Ammunition:", 24, 68, colors::white);
			global::abuffer->textwrite(i2str(ammo[0]) + " bullets, " + i2str(ammo[1]) + " rockets, " + i2str(ammo[2]) + " mines", 24, 70, colors::white);
			
			global::abuffer->textwrite("Points: " + i2str(points), 24, 72, colors::white);
			
			if(timeLimit >= 0)
				global::abuffer->textwrite("Time left: " + i2str(timeLimit >> 5) + "s", 48, 72, colors::white);
			else
				global::abuffer->textwrite("Time left: 0s", 48, 72, colors::white);
			
			if(nextStop != events.end())
			{
				global::abuffer->textwriteright(string("Next event: ") + stopNames[nextStopType], 98, 68, colors::white);
				if(breakNow)
				{
					global::abuffer->textwriteright("Break now to stop there!", 98, 70, colors::red);
				}
			}
			
			//Draw background
			int bgDrawPos = pos % 100;
			bg.draw(global::abuffer, bgDrawPos, 0, 0, 0, 100 - bgDrawPos, 67);
			if(bgDrawPos)
				bg.draw(global::abuffer, 0, 0, 100 - bgDrawPos, 0, bgDrawPos, 67);
			
			//Draw objects and enemies
			for(objIter i = objects.begin(); i != objects.end(); ++i)
			{
				object *curObj = *i;
				if(curObj->get_posx() + curObj->get_pic()->getW() < pos)
				{
					delete curObj;
					i = objects.erase(i);
					--i;
				}
				if(curObj->get_posx() < pos + 100)
					curObj->get_pic()->draw(global::abuffer, curObj->get_posx() - pos, curObj->get_posy());
			}
			
			for(objIter i = events.begin(); i != events.end(); ++i)
			{
				object *curObj = *i;
				if(curObj->get_posx() + curObj->get_pic()->getW() < pos)
				{
					//Create memory leak to avoid program crash
					//delete curObj;
					i = events.erase(i);
					--i;
				}
				if(curObj->get_posx() < pos + 100)
					curObj->get_pic()->draw(global::abuffer, curObj->get_posx() - pos, 37 - curObj->get_pic()->getH());
			}
			
			for(enemIter i = enemies.begin(); i != enemies.end(); ++i)
			{
				enemy *curObj = *i;
				if(curObj->get_posx() < pos + 100)
					curObj->get_pic()->draw(global::abuffer, curObj->get_posx() - pos, curObj->get_posy());
			}
			
			train.draw(global::abuffer, 30, 33);
			global::abuffer->setPixel(33 + weaponPos, 34, 'O', colors::white);
			global::abuffer->setPixel(32 + weaponPos + cannonDrawPos[aim][0], 33 + cannonDrawPos[aim][1], cannonGfx[aim], colors::white);
			
			for(bulIter i = bullets.begin(); i != bullets.end(); ++i)
			{
				bullet *curObj = *i;
				if(curObj->get_damage() == 13 || curObj->get_damage() == 25)
				{
					global::abuffer->setPixel(curObj->get_posx() - pos, curObj->get_posy(), cannonGfx[curObj->get_facing()], colors::white);
				}
				else
					curObj->get_pic()->draw(global::abuffer, curObj->get_posx() - pos, curObj->get_posy());
			}
			
			global::abuffer->draw(global::dbuffer);
			vsync();
			show();

			--timerCounter;
			if(timerCounter > 3)
				timerCounter = 0;
		}
		rest(1);
	}
	
	for(int i = 0; i < 255; ++i)
	{
		if(oblueprints[i])
			delete oblueprints[i];
		if(eblueprints[i])
			delete eblueprints[i];
	}
	for(objIter i = objects.begin(); i != objects.end(); )
	{
		delete *i;
		i = objects.erase(i);
	}
	for(objIter i = events.begin(); i != events.end(); )
	{
		delete *i;
		i = events.erase(i);
	}
	for(enemIter i = enemies.begin(); i != enemies.end(); )
	{
		delete *i;
		i = enemies.erase(i);
	}

	//Show outro animation and points screen
	if(levelFinished)
	{
		int bgDrawPos = pos % 100;
		pos = 70;
		speed = speedIncrement[speed];
		while(pos)
		{
			if(timerCounter > 0)
			{
				speedCounter += 3;
				while(speedCounter >= speed)
				{
					--pos;
					speedCounter -= speed;
				}
				clear_to_color(global::dbuffer, colors::black);
				bg.draw(global::abuffer, bgDrawPos, 0, 0, 0, 100 - bgDrawPos, 75);
				if(bgDrawPos)
					bg.draw(global::abuffer, 0, 0, 100 - bgDrawPos, 0, bgDrawPos, 75);
				train.draw(global::abuffer, 100 - pos, 33);
				global::abuffer->setPixel(103 + weaponPos - pos, 34, 'O', colors::white);
				global::abuffer->setPixel(102 + weaponPos + cannonDrawPos[aim][0] - pos, 33 + cannonDrawPos[aim][1], cannonGfx[aim], colors::white);
				global::abuffer->draw(global::dbuffer);
				show();
				if(keyStates[KEY_ESC])
					pos = 0;
				--timerCounter;
			}
			rest(1);
		}
		if(timeLimit < 0)
			timeLimit /= 2;
		points += timeLimit / 32 * 10;
		points -= stationsMissed * 200;
		missionCompleted(points, level + 1, oldScore, enemiesKilled, timeLimit / 32, stationsMissed);
	}
	if(playerHealth <= 0 || playerFuel <= 0)
	{
		/* DONE (any#1#): Show "Game over" screen */
		gameOver(playerHealth, playerFuel, points);
	}
	//Return false if the player died
	return levelFinished;
}

void saveLevel(int level)
{
	ifstream otherFile("progress.sav", ios::binary);
	if(!otherFile.fail())
	{
		int oldLevel;
		otherFile.read(reinterpret_cast<char*>(&oldLevel), sizeof(oldLevel));
		if(oldLevel >= level)
			return;
		otherFile.close();
	}
	ofstream file("progress.sav", ios::binary);
	if(!file.fail())
	{
		file.write(reinterpret_cast<char*>(&level), sizeof(level));
	}
}
