/*
  Name: main.cpp
  Copyright: Polecatsoft 2005
  Author: Master Lenman
  Date: 11.08.05 14:44
  Description: This file contains the core functions of the program:
		main(), init() and uninit() as well as timer routines
  Game: SH '05 template
*/

#include "StdAfx.h"
#include <ctime>
using std::time;
#include "menues.h"
#include "asciibmp.h"
#include "game.h"

//Some prototypes
void init();
void uninit();
int mainmenu(bool finalize);

//The core function of the application
int main(int argc, char **argv)
{
	init();
	clear(global::dbuffer);
	int todo;
	while((todo = mainmenu(false)) != 3)
	{
		if(todo == 0)
		{
			/* DONE (Lenman#1#): Add call to game() function */
			int level = -1, points = 0;
			level = selectLevel();
			if(level < 0)
				continue;
			while(true)
			{
				showBriefing(level);
				if(!game(level, points))
					break;
				++level;
				if(!fileExists(std::string("maps/map") + i2str(level) + ".map"))
				{
					todo = 2;
					/* DONE (any#1#): Show congrats using points */
					--level;
					break;
				}
				if(level == 4)
					points = 0;
			}
			if(level > 0)
				saveLevel(level);
		}
		if(todo == 1)
		{
			/* DONE (any#1#): show manual */
			showManual();
		}
		if(todo == 2)
		{
			/* DONE (any#1#): show credits */
			showCredits();
		}
	}
	uninit();
	return 0;
} END_OF_MAIN()

asciibmp *mainmenuOptions[8];

int mainmenu(bool finalize)
{
	if(finalize)
	{
		for(int i = 0; i < 8; ++i)
		{
			delete mainmenuOptions[i];
		}
		return 0;
	}
	else
	{
		for(int i = 0; i < 8; ++i)
		{
			if(mainmenuOptions[i])
				break;
			mainmenuOptions[i] = new asciibmp(std::string("gfx/mainmenu") + i2str(i));
		}
		int curSelection = 0;
		bool firstTime = true;
		while(true)
		{
			if(timerCounter > 0)
			{
				if(keyStates[KEY_ENTER])
				{
					return curSelection;
				}
				bool redraw = false;
				if(keyStates[KEY_UP])
				{
					if(--curSelection < 0)
						curSelection = 3;
					redraw = true;
				}
				if(keyStates[KEY_DOWN])
				{
					if(++curSelection > 3)
						curSelection = 0;
					redraw = true;
				}
				if(redraw || firstTime)
				{
					firstTime = false;
					clear(global::dbuffer);
					global::abuffer->clear(colors::black);
					for(int i = 0; i < 4; ++i)
					{
						mainmenuOptions[i * 2 + (curSelection == i)]
							->draw(global::abuffer, (100 - mainmenuOptions[i * 2 + (curSelection == i)]->getW()) >> 1, 16 + 11 * i);
					}
					static std::string CREDITSMSG = "A game by Deathlord and Master Lenman for TINS '06";
					global::abuffer->textwrite(CREDITSMSG, 98 - CREDITSMSG.length(), 74, colors::white);
					global::abuffer->draw(global::dbuffer);
				}
				show();
				--timerCounter;
			}
			rest(1);
		}
	}
}

//timerCounterUpdater() will update timerCounter 32 times per second.
//This timer is used to make the game run at the same speed on all computers.
volatile int timerCounter = 0;
void timerCounterUpdater()
{
	timerCounter++;
} END_OF_STATIC_FUNCTION(timerCounterUpdater)

//initialise allegro and some globals
void init()
{
	//initialise Allegro, timer, keyboard and sound
	allegro_init();

	if(install_timer() < 0)
		error::timerError();
	if(install_keyboard() < 0)
		error::keyboardError();
	if(install_sound(DIGI_AUTODETECT, MIDI_AUTODETECT, NULL) < 0)
		error::soundDriverError();

	/* DONE (Deathlord#2#): Paste Game's name */
	//Set a title for the window
	set_window_title("TraIN Shooter");

	/* DONE (Deathlord#1#): Define color depth and resolution */
	set_color_depth(16);
	if(set_gfx_mode(GFX_AUTODETECT, 800, 600, 0, 0) < 0)
	{
		error::graphicModeError();
	}

	global::dbuffer = createBitmap(SCREEN_W, SCREEN_H);
	clear(global::dbuffer);
	textout_centre_ex(global::dbuffer, font, "Loading engine, please wait...", SCREEN_W >> 1, SCREEN_H >> 1, makecol(0xFF, 0xFF, 0xFF), -1);
	show();

    //Lock Timer Function and variable
	LOCK_FUNCTION(timerCounterUpdater);
	LOCK_VARIABLE(timerCounter);

    //Set timer to 32 fps
    /* DONE (Deathlord#1#): Adjust timer frequency */
	install_int_ex(timerCounterUpdater, BPS_TO_TIMER(32));

	//Set Charset
	set_uformat(U_ASCII);

	//Initialise rand with the time and the randomly picked number 0xABCDE
	srand(time(0) * 0xABCDE);
	
	colors::fill();
	
	global::abuffer = new asciibmp(100, 75);
}

void uninit()
{
	destroy_bitmap(global::dbuffer);
	mainmenu(true);
	delete global::abuffer;
}

//Display the doublebuffer on the screen
void show()
{
	blit(global::dbuffer, screen, 0, 0, 0, 0, SCREEN_W, SCREEN_H);
}
