#include "StdAfx.h"
#include "menues.h"
#include <fstream>

using namespace std;

int selectLevel()
{
	int maxlevel;
	if(!fileExists("progress.sav"))
		maxlevel = 5; 
	else
	{
		ifstream file("progress.sav", ios::in | ios::binary);
		file.read((char*)&maxlevel, sizeof(int));
		file.close();
		++maxlevel;
	}
	
	if(maxlevel < 5)
		maxlevel = 5;
	
	asciibmp header("gfx/levelselect");
	int selected = maxlevel - 1;
	if(maxlevel == 5)
		selected = 0;
	
	while(keyStates[KEY_ESC] == 0)
	{
		if(timerCounter > 0)
		{
			clear(global::dbuffer);
			global::abuffer->clear(colors::black);
			
			header.draw(global::abuffer, 50 - header.getW() / 2, 8);
			
			if(keyStates[KEY_ENTER] != 0)
			{
				return selected;
			}
			if(keyStates[KEY_UP] != 0)
			{
				if(selected == 0)
				{
					selected = maxlevel - 1;
				}
				else
				{
					--selected;
				}				
			}
			if(keyStates[KEY_DOWN] != 0)
			{
				if(selected == maxlevel - 1)
				{
					selected = 0;
				}
				else
				{
					++selected;
				}	
			}
			
			for(int i = 0; i < maxlevel; ++i)
			{
				string name;
				if(i < 4)
					name = "Tutorial level " + i2str(i + 1);
				else
					name = "Level " + i2str(i - 3);
				if(i != selected)
				{
					global::abuffer->textwritecenter(name, 50, 22 + 2 * i, colors::white);
				}
				else
				{
					global::abuffer->textwritecenter("--> " + name, 50, 22 + 2 * i, colors::red);
				}
			}

			global::abuffer->draw(global::dbuffer);
			show();
			--timerCounter;
		}
		rest(1);
	}
	return -1;
}

void showBriefing(int level)
{
	ifstream lvl((string("maps/map") + i2str(level) + ".brf").c_str(), ios::in | ios::binary);
	if(!lvl)
	{
		error::mapLoadError(string("maps/map") + i2str(level) + ".brf", "File does not exist!");
	}
	
	clear(global::dbuffer);
	global::abuffer->clear(colors::black);
	
	string s;
	int i = -1;
	while(true)
	{
		if(lvl.eof())
			break;		
		getline(lvl, s, '\n');
		++i;
		if(s[s.length() - 1] == '\r' || s[s.length() - 1] == '\n')
			s.erase(s.length() - 1);
		global::abuffer->textwrite(s, 0, i, colors::white);
	}
	lvl.close();
	
	global::abuffer->draw(global::dbuffer);
	show();	
	
	while(true)
	{
		if(timerCounter > 0)
		{
			--timerCounter;
			if(keyStates[KEY_ESC] != 0 || keyStates[KEY_ENTER] != 0 || keyStates[KEY_SPACE] != 0)
				return;
		}
		rest(1);
	}	
}

void showManual()
{
	const int page_num = 2;
	int page = 0;
	bool needs_redraw = true;
	
	while(keyStates[KEY_ESC] == 0 && keyStates[KEY_ENTER] == 0 && keyStates[KEY_SPACE] == 0)
	{
		if(timerCounter > 0)
		{
			if(keyStates[KEY_RIGHT] != 0)
			{
				if(page == page_num - 1)
				{
					page = 0;
				}
				else
				{
					++page;
				}
				needs_redraw = true;
			}
			if(keyStates[KEY_LEFT] != 0)
			{
				if(page == 0)
				{
					page = page_num - 1;	
				}
				else
				{
					--page;	
				}
				needs_redraw = true;
			}
			--timerCounter;
		}
		
		if(needs_redraw == true)
		{
			clear(global::dbuffer);
			global::abuffer->clear(colors::black);
	
			ifstream file((string("manuals/page") + i2str(page) + ".txt").c_str(), ios::in | ios::binary);
			if(!file)
			{
				error::fatalError(string("manuals/page") + i2str(page) + ".txt", "File does not exist!");
			}
			
			string s;
			int i = -1;
			while(true)
			{
				if(file.eof())
					break;				
				getline(file, s, '\n');
				++i;
				if(s[s.length() - 1] == '\r' || s[s.length() - 1] == '\n')
					s.erase(s.length() - 1);
				global::abuffer->textwrite(s, 0, i, colors::white);
			}
			file.close();
	
			global::abuffer->draw(global::dbuffer);
			show();	
			needs_redraw = false;
		}
		rest(1);
	}
}

void gameOver(int health, int fuel, int points)
{
	ifstream file("manuals/gameover.txt", ios::in | ios::binary);
	if(!file)
	{
		error::fatalError("A file couldn't be openend.", true);
	}
	
	clear(global::dbuffer);
	global::abuffer->clear(colors::black);
	
	string s;
	int i = -1;
	while(true)
	{
		if(file.eof())
			break;		
		getline(file, s, '\n');
		++i;
		if(s[s.length() - 1] == '\r' || s[s.length() - 1] == '\n')
			s.erase(s.length() - 1);
		global::abuffer->textwrite(s, 0, i, colors::white);
	}
	file.close();
	
	string msg;
	if(health <= 0)
	{
		msg = "Your train has been destroyed. You achieved " + i2str(points) + " points. Try again!";
	}
	else
	{
		msg = "You ran out of fuel in enemy lands. You achieved " + i2str(points) + " points. Try again!";
	}
	global::abuffer->textwritecenter(msg.c_str(), 50, 30, colors::white);
	global::abuffer->textwritecenter("-- Press space to continue --", 50, 72, colors::white);
	
	global::abuffer->draw(global::dbuffer);
	show();
	
	while(true)
	{
		if(timerCounter > 0)
		{
			if(keyStates[KEY_ESC] != 0 || keyStates[KEY_ENTER] != 0 || keyStates[KEY_SPACE] != 0)
				return;	
			--timerCounter;
		}
		rest(1);
	}		
}

void missionCompleted(int points, int level, int oldpoints, int kills, int timeleft, int stationsmissed)
{
	ifstream file("manuals/levelwon.txt", ios::in | ios::binary);
	if(!file)
	{
		error::fatalError("A file couldn't be openend.", true);
	}
	
	clear(global::dbuffer);
	global::abuffer->clear(colors::black);
	
	string s;
	int i = -1;
	while(true)
	{
		if(file.eof())
			break;		
		getline(file, s, '\n');
		++i;
		if(s[s.length() - 1] == '\r' || s[s.length() - 1] == '\n')
			s.erase(s.length() - 1);
		global::abuffer->textwrite(s, 0, i, colors::white);
	}
	file.close();
	
	global::abuffer->textwrite("Points achieved: ", 15, 19, colors::white);
	global::abuffer->textwrite("Enemy kills: " + i2str(kills) + " (* 5) -->", 15, 25, colors::white);
	global::abuffer->textwrite(i2str(kills * 5), 74 - i2str(kills * 5).length(), 25, colors::white);
	global::abuffer->textwrite("Time left: " + i2str(timeleft) + " (* 10) -->", 15, 27, colors::white);
	global::abuffer->textwrite(i2str(timeleft * 10), 74 - i2str(timeleft * 10).length(), 27, colors::white);
	global::abuffer->textwrite("Stations missed: " + i2str(stationsmissed) + " (* -200) -->", 15, 29, colors::white);
	global::abuffer->textwrite(i2str(stationsmissed * (-200)), 74 - i2str(stationsmissed * (-200)).length(), 29, colors::white);
	global::abuffer->textwrite("Points from previous missions :", 15, 31, colors::white);
	global::abuffer->textwrite(i2str(oldpoints), 74 - i2str(oldpoints).length(), 31, colors::white);
	global::abuffer->textwrite("-----------------------------------------------------------", 15, 33, colors::white);
	global::abuffer->textwrite("Total points:", 15, 35, colors::white);
	global::abuffer->textwrite(i2str(points), 74 - i2str(points).length(), 35, colors::white);
	global::abuffer->textwritecenter("-- Press space to start level " + i2str(level + 1) + " --", 50, 72, colors::white);
	
	global::abuffer->draw(global::dbuffer);
	show();	
	
	while(true)
	{
		if(timerCounter > 0)
		{
			if(keyStates[KEY_ESC] != 0 || keyStates[KEY_ENTER] != 0 || keyStates[KEY_SPACE] != 0)
				return;	
			--timerCounter;
		}
		rest(1);
	}			
}

void showCredits()
{
	ifstream file("manuals/credits.txt", ios::in | ios::binary);
	if(!file)
	{
		error::fatalError("A file couldn't be openend.", true);
	}
	
	clear(global::dbuffer);
	global::abuffer->clear(colors::black);
	
	string s;
	int i = -1;
	while(true)
	{
		if(file.eof())
			break;		
		getline(file, s, '\n');
		++i;
		if(s[s.length() - 1] == '\r' || s[s.length() - 1] == '\n')
			s.erase(s.length() - 1);
		global::abuffer->textwrite(s, 0, i, colors::white);
	}
	file.close();
	
	global::abuffer->draw(global::dbuffer);
	show();	
	
	while(true)
	{
		if(timerCounter > 0)
		{
			if(keyStates[KEY_ESC] != 0 || keyStates[KEY_ENTER] != 0 || keyStates[KEY_SPACE] != 0)
				return;	
			--timerCounter;
		}
		rest(1);
	}		
}
